package segmentation;

import gate.gui.*;
import gate.*;
import javax.swing.*;
import java.awt.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.awt.event.*;

/**
 * Modified version DocumentEditor in order to add some access to private
 * method or object in the default version of this class. Add a search and a
 * replace function. You need to modify the creole.xml file to tell Gate to
 * use this document editor instead of the default one.
 */
public class DocumentEditor extends gate.gui.docview.DocumentEditor {

    /** The dialog used for text search */
    private SearchDialog searchDialog;

    /** The dialog used for annotation replace */
    private ReplaceDialog replaceDialog;

    protected void initViews() {
        super.initViews();
        topBar.add(Box.createHorizontalStrut(5));
        topBar.add(new SearchAction());
        topBar.add(Box.createHorizontalStrut(5));
        topBar.add(new ReplaceAction());
    }

      /**
       * Do a click on a toggle button of the top button bar.
       *
       * @param name text displayed on the button
       */
      public void activeButton(String name) {

          if (topBar == null) { return; }

          for (int k = 0; k < topBar.getComponentCount(); k++) {
              if (topBar.getComponent(k) instanceof JToggleButton) {

                  JToggleButton jtb = (JToggleButton)topBar.getComponent(k);
                  if (jtb.getText().equals(name) && !jtb.isSelected()) {
                      jtb.doClick();
                  }
              }
          }
      }

      public Document getDocument() {
          return document;
      }

      /**
       * Return the table model of annotation sets view based on
       * AnnotationSetsView.initGUI().
       *
       * FIXME: very awkwardly code.
       *
       * @return table model of annotation sets view
       */
      public javax.swing.table.TableModel getAnnotationTableModel() {

          javax.swing.table.TableModel annotationTableModel = null;
          gate.gui.docview.AnnotationSetsView asv = 
              (gate.gui.docview.AnnotationSetsView)getRightView();

          if (asv == null) { return annotationTableModel; }

          annotationTableModel = (javax.swing.table.TableModel)
              ((gate.swing.XJTable)
               ((javax.swing.JViewport)
                ((JScrollPane)((javax.swing.JPanel)asv.getGUI())
                 .getComponent(0)).getViewport()).getView()).getModel();

          return annotationTableModel;
      }

      /**
       * Expand the first annotation set.
       *
       * FIXME: very awkwardly code.
       */
      public void expandAnnotationTable() {

          javax.swing.table.TableModel annotationTableModel =
              getAnnotationTableModel();

          if (annotationTableModel != null) {
              annotationTableModel.setValueAt(new Boolean(true),0,0);
          }
      }

      /**
       * Tick or untick annotation check box in annotation table of the
       * right view.
       */
      public void setTypeSelected(String set, String type,
                                  boolean ticked) {

          gate.gui.docview.AnnotationSetsView asv =
              (gate.gui.docview.AnnotationSetsView)getRightView();

          if (asv == null ||
              !document.getAnnotations().getAllTypes().contains(type)) {

              return;
          }

          asv.setTypeSelected(set, type, ticked);
      }

      /**
       * Code taken from gate.gui.DocumentEditor, the old DocumentEditor.
       * Modified to work with gate 3.1.
       */
      protected class SearchAction extends AbstractAction {

          public SearchAction() {
              super("Search text");
              putValue(SHORT_DESCRIPTION, "Search within the text");
          }

          public void actionPerformed(ActionEvent evt) {
              if (searchDialog == null) {
                  Window parent =
                      SwingUtilities.getWindowAncestor(DocumentEditor.this);
                  searchDialog = (parent instanceof Dialog) ?
                      new SearchDialog((Dialog)parent) :
                      new SearchDialog((Frame)parent);
                  searchDialog.pack();
                  searchDialog.setLocationRelativeTo(DocumentEditor.this);
                  searchDialog.setResizable(false);
                  MainFrame.getGuiRoots().add(searchDialog);
              }

              if (searchDialog.isVisible()) {
                  searchDialog.toFront();
                  searchDialog.patternTextField.selectAll();
              } else {
                  searchDialog.setVisible(true);
                  searchDialog.patternTextField.selectAll();
              }
          }
      }

      protected class SearchDialog extends JDialog {

          SearchDialog(Frame owner) {
              super(owner, false);
              setTitle("Find in \"" + document.getName() + "\"");
              initLocalData();
              initGuiComponents();
              initListeners();
          }

          SearchDialog(Dialog owner) {
              super(owner, false);
              setTitle("Find in \"" + document.getName() + "\"");
              initLocalData();
              initGuiComponents();
              initListeners();
          }

          protected void initLocalData() {
              pattern = null;
              nextMatchStartsFrom = 0;
              content = document.getContent().toString();

              findFirstAction = new AbstractAction("Find first") {
                      {
                          putValue(SHORT_DESCRIPTION, "Finds first match");
                      }

                public void actionPerformed(ActionEvent evt) {
                    //needed to create the right RE
                    refresh();
                    //remove selection
                    textPane.setCaretPosition(textPane.getCaretPosition());
                    boolean found = false;
                    int start = -1;
                    int end = -1;
                    nextMatchStartsFrom = 0;

                    Matcher matcher = pattern.matcher(content);
                    while (matcher.find(nextMatchStartsFrom)
                           && (found == false)) {
                        start = matcher.start();
                        end = matcher.end();
                        if (wholeWordsChk.isSelected()) {
                            //validate the result
                            found = (start == 0
                            || !Character.isLetterOrDigit(
                                     content.charAt(start - 1)))
                                && (end == content.length()
                                    || !Character.isLetterOrDigit(
                                                     content.charAt(end)));
                        } else {
                            found = true;
                        }
                        nextMatchStartsFrom = start + 1;
                    }

                    if (found) {
                        //display the result
                        textPane.setCaretPosition(start);
                        textPane.moveCaretPosition(end);
                        textPane.requestFocusInWindow();
                        SwingUtilities.getWindowAncestor(textPane)
                            .toFront();

                    } else {
                        JOptionPane.showMessageDialog(searchDialog,
                          "String not found!", "GATE",
                          JOptionPane.INFORMATION_MESSAGE);
                    }
                }};

              findNextAction = new AbstractAction("Find next") {
                      {
                          putValue(SHORT_DESCRIPTION, "Finds next match");
                      }
                public void actionPerformed(ActionEvent evt) {
                    //needed to create the right RE
                    refresh();
                    //remove selection
                    textPane.setCaretPosition(textPane.getCaretPosition());
                    boolean found = false;
                    int start = -1;
                    int end = -1;

                    Matcher matcher = pattern.matcher(content);
                    while (matcher.find(nextMatchStartsFrom)
                           && (found == false)) {
                        start = matcher.start();
                        end = matcher.end();
                        if (wholeWordsChk.isSelected()) {
                            //validate the result
                            found = (start == nextMatchStartsFrom
                            || !Character.isLetterOrDigit(
                                     content.charAt(start - 1)))
                                && (end == content.length()
                                    || !Character.isLetterOrDigit(
                                                     content.charAt(end)));
                        } else {
                            found = true;
                        }
                        nextMatchStartsFrom = start + 1;
                    }

                    if (found) {
                        //display the result
                        textPane.setCaretPosition(start);
                        textPane.moveCaretPosition(end);
                        textPane.requestFocusInWindow();

                    } else {
                        JOptionPane.showMessageDialog(searchDialog,
                          "String not found!", "GATE",
                          JOptionPane.INFORMATION_MESSAGE);
                    }
                }};

              cancelAction = new AbstractAction("Cancel") {
                      {
                          putValue(SHORT_DESCRIPTION, "Cancel");
                      }
                      public void actionPerformed(ActionEvent evt){
                          searchDialog.setVisible(false);
                      }
                  };
          }

          protected void initGuiComponents() {
              getContentPane().setLayout(new BoxLayout(getContentPane(),
                                                       BoxLayout.Y_AXIS));

              getContentPane().add(Box.createVerticalStrut(5));

              Box hBox = Box.createHorizontalBox();
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(new JLabel("Find what:"));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(patternTextField = new JTextField(20));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(Box.createHorizontalGlue());
              getContentPane().add(hBox);

              getContentPane().add(Box.createVerticalStrut(5));

              hBox = Box.createHorizontalBox();
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(ignoreCaseChk = new JCheckBox("Ignore case", false));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(wholeWordsChk =
                       new JCheckBox("Whole words", false));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(regularExpressionChk =
                       new JCheckBox("Regular Expression", false));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(Box.createHorizontalGlue());
              getContentPane().add(hBox);

              getContentPane().add(Box.createVerticalStrut(5));

              hBox = Box.createHorizontalBox();
              hBox.add(Box.createHorizontalGlue());
              hBox.add(new JButton(findFirstAction));
              hBox.add(Box.createHorizontalStrut(5));
              findNextJButton = new JButton(findNextAction);
              hBox.add(findNextJButton);
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(new JButton(cancelAction));
              hBox.add(Box.createHorizontalGlue());
              getContentPane().add(hBox);

              getContentPane().add(Box.createVerticalStrut(5));

              getRootPane().setDefaultButton(findNextJButton);
          }

          protected void initListeners() {

              addComponentListener(new ComponentAdapter() {
                      public void componentHidden(ComponentEvent e) {
                      }

                      public void componentMoved(ComponentEvent e) {
                      }

                      public void componentResized(ComponentEvent e) {
                      }

                      public void componentShown(ComponentEvent e) {
                          refresh();
                      }
                  });

              patternTextField.getDocument().addDocumentListener(
                new javax.swing.event.DocumentListener() {
                    public void insertUpdate(javax.swing.event
                                             .DocumentEvent e) {
                        refresh();
                    }

                    public void removeUpdate(javax.swing.event
                                             .DocumentEvent e) {
                        refresh();
                    }

                    public void changedUpdate(javax.swing.event
                                              .DocumentEvent e) {
                        refresh();
                    }
                });

          }

          protected void refresh() {
              String patternText = patternTextField.getText();

              if (patternText != null && patternText.length() > 0) {
                  //update actions state
                  findFirstAction.setEnabled(true);
                  findNextAction.setEnabled(true);

                  //update patternRE
                  try {
                      if (regularExpressionChk.isSelected()) {
                          pattern = ignoreCaseChk.isSelected() ?
                              Pattern.compile(patternText,
                                              Pattern.CASE_INSENSITIVE) :
                              Pattern.compile(patternText);
                      } else {

                          pattern = ignoreCaseChk.isSelected() ?
                              Pattern.compile("\\Q"+patternText+"\\E",
                                              Pattern.CASE_INSENSITIVE) :
                              Pattern.compile("\\Q"+patternText+"\\E");
                      }

                  } catch (Exception ree) {
                      JOptionPane.showMessageDialog(searchDialog,
                        "Invalid pattern!\n" + ree.toString(), "GATE",
                        JOptionPane.ERROR_MESSAGE);
                  }

              } else {
                  findFirstAction.setEnabled(false);
                  findNextAction.setEnabled(false);
              }

              if (pattern == null) {}
          }

          // FIXME
          javax.swing.text.JTextComponent textPane =
              (javax.swing.text.JTextComponent)
              ((javax.swing.JViewport)
               ((JScrollPane)getCentralView().getGUI())
               .getViewport()).getView();

          JTextField patternTextField;
          JCheckBox ignoreCaseChk;
          JCheckBox wholeWordsChk;
          JCheckBox regularExpressionChk;
          Pattern pattern;
          int nextMatchStartsFrom;
          String content;
          Action findFirstAction;
          Action findNextAction;
          Action cancelAction;
          JButton findNextJButton;

      } // end of class SearchDialog

      /**
       * Replace an annotation type with another type and keep the features.
       */
      protected class ReplaceAction extends AbstractAction {

          public ReplaceAction() {
              super("Search & Replace annotations");
              putValue(SHORT_DESCRIPTION,
                       "Search and Replace annotation with another one");
          }

          public void actionPerformed(ActionEvent evt) {
              if (replaceDialog == null) {
                  Window parent =
                      SwingUtilities.getWindowAncestor(DocumentEditor.this);
                  replaceDialog = (parent instanceof Dialog) ?
                      new ReplaceDialog((Dialog)parent) :
                      new ReplaceDialog((Frame)parent);
                  replaceDialog.pack();
                  replaceDialog.setLocationRelativeTo(DocumentEditor.this);
                  replaceDialog.setResizable(false);
                  MainFrame.getGuiRoots().add(replaceDialog);
              }

              if (replaceDialog.isVisible()) {
                  replaceDialog.toFront();
                  replaceDialog.patternTextField.selectAll();
              } else {
                  replaceDialog.setVisible(true);
                  replaceDialog.patternTextField.selectAll();
              }
          }
      }

      protected class ReplaceDialog extends JDialog {

          ReplaceDialog(Frame owner) {
              super(owner, false);
              setTitle("Replace in \"" + document.getName() + "\"");
              initLocalData();
              initGuiComponents();
              initListeners();
          }

          ReplaceDialog(Dialog owner) {
              super(owner, false);
              setTitle("Replace in \"" + document.getName() + "\"");
              initLocalData();
              initGuiComponents();
              initListeners();
          }

          protected void initLocalData() {
              pattern = null;
              nextMatchStartsFrom = 0;
              content = document.getContent().toString();
              annotationSelected = null;

              findFirstAction = new AbstractAction("Find first") {
                      {
                          putValue(SHORT_DESCRIPTION, "Finds first match");
                      }
                public void actionPerformed(ActionEvent evt) {
                    AnnotationSet AS = document.getAnnotations();
                    gate.AnnotationSet aAS;
                    gate.Annotation A;
                    int start = -1;
                    int end = -1;
                    String s;
                    Object[] array;
                    boolean found = false;

                    //needed to create the right RE
                    refresh();
                    //remove selection
                    textPane.setCaretPosition(textPane.getCaretPosition());
                    annotationSelected = null;

                    // get the annotations set to scan
                    aAS =AS.get((String)annotationToFind.getSelectedItem());
                    if (aAS != null) {
                    array = aAS.toArray();
                    java.util.Comparator comparator =
                        new java.util.Comparator() {
                            public int compare(Object o1, Object o2) {
                                SimpleAnnotation sa1 = (SimpleAnnotation)o1;
                                SimpleAnnotation sa2 = (SimpleAnnotation)o2;
                                if (sa1.getStartNode().getOffset().equals(
                                    sa2.getStartNode().getOffset())) {
                                    return sa1.getId().compareTo(
                                           sa2.getId());
                                } else {
                                    return sa1.getStartNode().getOffset()
                                .compareTo(sa2.getStartNode().getOffset());
                                }
                            }
                        };
                    // sort annotations in the same way as text
                    java.util.Arrays.sort(array, comparator);
                    for (int i = 0; i < array.length ; i++) {

                        A = (gate.Annotation)array[i];
                        start = A.getStartNode().getOffset().intValue();
                        end = A.getEndNode().getOffset().intValue();
                        s = content.substring(start, end);

                        if (pattern == null || pattern.matcher(s).find()) {
                            found = true;
                            annotationSelected = A;
                            nextMatchStartsFrom = i+1;
                            break;
                        }
                    }
                    }

                    if (found) {
                        // select the result
                        textPane.setCaretPosition(start);
                        textPane.moveCaretPosition(end);
                        textPane.requestFocusInWindow();
                        SwingUtilities.getWindowAncestor(textPane)
                            .toFront();
                        refresh();

                    } else {
                        JOptionPane.showMessageDialog(replaceDialog,
                          "Annotation not found!", "GATE",
                          JOptionPane.INFORMATION_MESSAGE);
                    }
                }};

              findNextAction = new AbstractAction("Find next") {
                      {
                          putValue(SHORT_DESCRIPTION, "Finds next match");
                      }
                public void actionPerformed(ActionEvent evt) {
                    AnnotationSet AS = document.getAnnotations();
                    gate.AnnotationSet aAS;
                    gate.Annotation A;
                    int start = -1;
                    int end = -1;
                    String s;
                    Object[] array;
                    boolean found = false;

                    //needed to create the right RE
                    refresh();
                    //remove selection
                    textPane.setCaretPosition(textPane.getCaretPosition());
                    annotationSelected = null;

                    // get the annotations set to scan
                    aAS =AS.get((String)annotationToFind.getSelectedItem());
                    if (aAS != null) {
                    array = aAS.toArray();
                    java.util.Comparator comparator =
                        new java.util.Comparator() {
                            public int compare(Object o1, Object o2) {
                                SimpleAnnotation sa1 = (SimpleAnnotation)o1;
                                SimpleAnnotation sa2 = (SimpleAnnotation)o2;
                                if (sa1.getStartNode().getOffset().equals(
                                    sa2.getStartNode().getOffset())) {
                                    return sa1.getId().compareTo(
                                           sa2.getId());
                                } else {
                                    return sa1.getStartNode().getOffset()
                                .compareTo(sa2.getStartNode().getOffset());
                                }
                            }
                        };
                    // sort annotations in the same way as text
                    java.util.Arrays.sort(array, comparator);
                    for (int i = nextMatchStartsFrom;
                         i < array.length ; i++) {

                        A = (gate.Annotation)array[i];
                        start = A.getStartNode().getOffset().intValue();
                        end = A.getEndNode().getOffset().intValue();
                        s = content.substring(start, end);

                        if (pattern == null || pattern.matcher(s).find()) {
                            found = true;
                            annotationSelected = A;
                            nextMatchStartsFrom = i+1;
                            break;
                        }
                    }
                    }

                    if (found) {
                        //display the result
                        textPane.setCaretPosition(start);
                        textPane.moveCaretPosition(end);
                        textPane.requestFocusInWindow();
                        refresh();

                    } else {
                        JOptionPane.showMessageDialog(replaceDialog,
                          "Annotation not found!", "GATE",
                          JOptionPane.INFORMATION_MESSAGE);
                    }
                }};

              replaceAction = new AbstractAction("Replace") {
                      {
                          putValue(SHORT_DESCRIPTION,
                                   "Replace annotations");
                      }
                      public void actionPerformed(ActionEvent evt){

                          AnnotationSet AS = document.getAnnotations();
                          AS.remove(annotationSelected);
                          AS.add(annotationSelected.getStartNode(),
                                 annotationSelected.getEndNode(),
                                 (String)annotationReplacement
                                 .getSelectedItem(),
                                 annotationSelected.getFeatures());

                          // highlight next match
                          if (nextMatchStartsFrom > 0) {
                              nextMatchStartsFrom -= 1;
                          }
                          findNextJButton.doClick();
                      }
                  };

              replaceAllAction = new AbstractAction("Replace all") {
                      {
                          putValue(SHORT_DESCRIPTION,
                                   "Replace all annotations");
                      }

                  public void actionPerformed(ActionEvent evt) {
                    AnnotationSet AS = document.getAnnotations();
                    gate.AnnotationSet aAS;
                    gate.Annotation A;
                    int start = -1;
                    int end = -1;
                    String s;
                    Object[] array;

                    //needed to create the right RE
                    refresh();
                    //remove selection
                    textPane.setCaretPosition(textPane.getCaretPosition());
                    annotationSelected = null;

                    // get the annotations set to scan
                    aAS =AS.get((String)annotationToFind.getSelectedItem());
                    if (aAS != null) {
                    array = aAS.toArray();
                    java.util.Comparator comparator =
                        new java.util.Comparator() {
                            public int compare(Object o1, Object o2) {
                                SimpleAnnotation sa1 = (SimpleAnnotation)o1;
                                SimpleAnnotation sa2 = (SimpleAnnotation)o2;
                                if (sa1.getStartNode().getOffset().equals(
                                    sa2.getStartNode().getOffset())) {
                                    return sa1.getId().compareTo(
                                           sa2.getId());
                                } else {
                                    return sa1.getStartNode().getOffset()
                                .compareTo(sa2.getStartNode().getOffset());
                                }
                            }
                        };
                    // sort annotations in the same way as text
                    java.util.Arrays.sort(array, comparator);
                    int num = 0;
                    for (int i = 0; i < array.length ; i++) {

                        A = (gate.Annotation)array[i];
                        start = A.getStartNode().getOffset().intValue();
                        end = A.getEndNode().getOffset().intValue();
                        s = content.substring(start, end);

                        if (pattern == null || pattern.matcher(s).find()) {
                            annotationSelected = A;
                            // replace annotation
                            AS.remove(annotationSelected);
                            AS.add(annotationSelected.getStartNode(),
                                   annotationSelected.getEndNode(),
                                   (String)annotationReplacement
                                   .getSelectedItem(),
                                   annotationSelected.getFeatures());
                            num++;
                        }
                    }
                    JOptionPane.showMessageDialog(replaceDialog,
                        num+" annotations replaced.", "GATE",
                        JOptionPane.INFORMATION_MESSAGE);
                    }
                    refresh();
                  }
              };

              cancelAction = new AbstractAction("Cancel") {
                      {
                          putValue(SHORT_DESCRIPTION, "Cancel");
                      }
                      public void actionPerformed(ActionEvent evt){
                          replaceDialog.setVisible(false);
                      }
                  };
          }

          protected void initGuiComponents() {

              getContentPane().setLayout(new BoxLayout(getContentPane(),
                                                       BoxLayout.Y_AXIS));

              getContentPane().add(Box.createVerticalStrut(5));

              Box hBox = Box.createHorizontalBox();
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(new JLabel("Find:"));
              hBox.add(Box.createHorizontalStrut(5));
              annotationToFind = new JComboBox();
              hBox.add(annotationToFind);
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(new JLabel("Replace with:"));
              hBox.add(Box.createHorizontalStrut(5));
              annotationReplacement = new JComboBox();
              hBox.add(annotationReplacement);
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(Box.createHorizontalGlue());
              getContentPane().add(hBox);

              getContentPane().add(Box.createVerticalStrut(5));

              hBox = Box.createHorizontalBox();
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(new JLabel("Text to find:"));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(patternTextField = new JTextField(10));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(Box.createHorizontalGlue());
              getContentPane().add(hBox);

              getContentPane().add(Box.createVerticalStrut(5));

              hBox = Box.createHorizontalBox();
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(ignoreCaseChk = new JCheckBox("Ignore case", false));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(wholeWordsChk =
                       new JCheckBox("Whole words", false));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(regularExpressionChk =
                       new JCheckBox("Regular Expression", false));
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(Box.createHorizontalGlue());
              getContentPane().add(hBox);

              getContentPane().add(Box.createVerticalStrut(5));

              hBox = Box.createHorizontalBox();
              hBox.add(Box.createHorizontalGlue());
              findFirstJButton = new JButton(findFirstAction);
              hBox.add(findFirstJButton);
              hBox.add(Box.createHorizontalStrut(5));
              findNextJButton = new JButton(findNextAction);
              hBox.add(findNextJButton);
              hBox.add(Box.createHorizontalStrut(5));
              replaceJButton = new JButton(replaceAction);
              hBox.add(replaceJButton);
              hBox.add(Box.createHorizontalStrut(5));
              replaceAllJButton = new JButton(replaceAllAction);
              hBox.add(replaceAllJButton);
              hBox.add(Box.createHorizontalStrut(5));
              hBox.add(new JButton(cancelAction));
              hBox.add(Box.createHorizontalGlue());
              getContentPane().add(hBox);

              getContentPane().add(Box.createVerticalStrut(5));

              getRootPane().setDefaultButton(findNextJButton);
          }

          protected void initListeners() {

              addComponentListener(new ComponentAdapter() {
                      public void componentHidden(ComponentEvent e) {
                      }

                      public void componentMoved(ComponentEvent e) {
                      }

                      public void componentResized(ComponentEvent e) {
                      }

                      public void componentShown(ComponentEvent e) {
                          updateAnnotationLists();
                          refresh();
                      }
                  });

              patternTextField.getDocument().addDocumentListener(
                new javax.swing.event.DocumentListener() {
                    public void insertUpdate(javax.swing.event
                                             .DocumentEvent e) {
                        refresh();
                    }

                    public void removeUpdate(javax.swing.event
                                             .DocumentEvent e) {
                        refresh();
                    }

                    public void changedUpdate(javax.swing.event
                                              .DocumentEvent e) {
                        refresh();
                    }
                });

              annotationToFind.addItemListener(
                new java.awt.event.ItemListener() {
                    public void itemStateChanged(ItemEvent e) {
                        if (!annotationToFind.getActionCommand()
                            .equals("disable")) {
                            refresh();
                        }
                    }
                });

              annotationReplacement.addItemListener(
                new java.awt.event.ItemListener() {
                    public void itemStateChanged(ItemEvent e) {
                        if (!annotationReplacement.getActionCommand()
                            .equals("disable")) {
                            refresh();
                        }
                    }
                });

              document.getAnnotations().addAnnotationSetListener(
                new gate.event.AnnotationSetListener() {
                    public void annotationAdded(gate.event
                                                .AnnotationSetEvent e) {
                        updateAnnotationLists();
                        refresh();
                    }
                    public void annotationRemoved(gate.event
                                                  .AnnotationSetEvent e) {
                        updateAnnotationLists();
                        refresh();
                    }
                });
          }

          protected void refresh() {

              if (annotationToFind.getSelectedItem() == null) {
                  findFirstAction.setEnabled(false);
                  findNextAction.setEnabled(false);
              } else {
                  findFirstAction.setEnabled(true);
                  findNextAction.setEnabled(true);
              }

              if (annotationSelected == null
                  || annotationReplacement.getSelectedItem() == null) {
                  replaceAction.setEnabled(false);
                  replaceAllAction.setEnabled(false);
              } else {
                  replaceAction.setEnabled(true);
                  replaceAllAction.setEnabled(true);
              }

              String patternText = patternTextField.getText().trim();

              if (patternText != null && patternText.length() > 0) {

                  //update patternRE
                  try {
                      if (regularExpressionChk.isSelected()) {
                          pattern = ignoreCaseChk.isSelected() ?
                              Pattern.compile(patternText,
                                              Pattern.CASE_INSENSITIVE) :
                              Pattern.compile(patternText);
                      } else {

                          pattern = ignoreCaseChk.isSelected() ?
                              Pattern.compile("\\Q"+patternText+"\\E",
                                              Pattern.CASE_INSENSITIVE) :
                              Pattern.compile("\\Q"+patternText+"\\E");
                      }

                  } catch (Exception ree) {
                      JOptionPane.showMessageDialog(replaceDialog,
                        "Invalid pattern!\n" + ree.toString(), "GATE",
                        JOptionPane.ERROR_MESSAGE);
                  }
              } else {
                  pattern = null;
              }
          }

          protected void updateAnnotationLists() {

              Object[] as;

              if (document.getAnnotations() == null) {
                  as = null;
              } else {
                  as = document.getAnnotations().getAllTypes().toArray();
              }

              java.util.Arrays.sort(as);
              Object previousSITF = annotationToFind.getSelectedItem();
              Object previousSIR = annotationReplacement.getSelectedItem();
              annotationToFind.setActionCommand("disable");
              annotationReplacement.setActionCommand("disable");
              annotationToFind.setModel(new DefaultComboBoxModel(as));
              annotationReplacement.setModel(new DefaultComboBoxModel(as));
              annotationToFind.setSelectedItem(previousSITF);
              annotationReplacement.setSelectedItem(previousSIR);
              annotationToFind.setActionCommand("enable");
              annotationReplacement.setActionCommand("enable");
          }

          // FIXME
          javax.swing.text.JTextComponent textPane =
              (javax.swing.text.JTextComponent)
              ((javax.swing.JViewport)
               ((JScrollPane)getCentralView().getGUI())
               .getViewport()).getView();

          JTextField patternTextField;
          JCheckBox ignoreCaseChk;
          JCheckBox wholeWordsChk;
          JCheckBox regularExpressionChk;
          Pattern pattern;
          int nextMatchStartsFrom;
          String content;
          Action findFirstAction;
          Action findNextAction;
          Action replaceAction;
          Action replaceAllAction;
          Action cancelAction;
          JButton findFirstJButton;
          JButton findNextJButton;
          JButton replaceJButton;
          JButton replaceAllJButton;
          JComboBox annotationToFind;
          JComboBox annotationReplacement;
          gate.Annotation annotationSelected;

      } // end of class ReplaceDialog

  } // end of class DocumentEditor
